@@ -25,6 +25,7 @@
 #include "pulse-connection.h"
 #include "pulse-helpers.h"
 #include "pulse-monitor.h"
+#include "pulse-stream.h"
 #include "pulse-stream-control.h"
 
 struct _PulseStreamControlPrivate
@@ -270,11 +271,25 @@ pulse_stream_control_finalize (GObject *object)
 guint32
 pulse_stream_control_get_index (PulseStreamControl *control)
 {
-    g_return_val_if_fail (PULSE_IS_STREAM_CONTROL (control), 0);
+    g_return_val_if_fail (PULSE_IS_STREAM_CONTROL (control), PA_INVALID_INDEX);
 
     return control->priv->index;
 }
 
+guint32
+pulse_stream_control_get_stream_index (PulseStreamControl *control)
+{
+    MateMixerStream *stream;
+
+    g_return_val_if_fail (PULSE_IS_STREAM_CONTROL (control), PA_INVALID_INDEX);
+
+    stream = mate_mixer_stream_control_get_stream (MATE_MIXER_STREAM_CONTROL (control));
+    if G_UNLIKELY (stream == NULL)
+        return PA_INVALID_INDEX;
+
+    return pulse_stream_get_index (PULSE_STREAM (stream));
+}
+
 PulseConnection *
 pulse_stream_control_get_connection (PulseStreamControl *control)
 {
@@ -308,14 +323,19 @@ pulse_stream_control_get_channel_map (PulseStreamControl *control)
 }
 
 void
-pulse_stream_control_set_app_info (PulseStreamControl *control, MateMixerAppInfo *info)
+pulse_stream_control_set_app_info (PulseStreamControl *control,
+                                   MateMixerAppInfo   *info,
+                                   gboolean            take)
 {
     g_return_if_fail (PULSE_IS_STREAM_CONTROL (control));
 
-    if G_UNLIKELY (control->priv->app_info)
+    if G_UNLIKELY (control->priv->app_info != NULL)
         _mate_mixer_app_info_free (control->priv->app_info);
 
-    control->priv->app_info = info;
+    if (take == TRUE)
+        control->priv->app_info = info;
+    else
+        control->priv->app_info = _mate_mixer_app_info_copy (info);
 }
 
 void
@@ -324,11 +344,10 @@ pulse_stream_control_set_channel_map (PulseStreamControl *control, const pa_chan
     MateMixerStreamControlFlags flags;
 
     g_return_if_fail (PULSE_IS_STREAM_CONTROL (control));
-    g_return_if_fail (map != NULL);
 
     flags = mate_mixer_stream_control_get_flags (MATE_MIXER_STREAM_CONTROL (control));
 
-    if (pa_channel_map_valid (map)) {
+    if (map != NULL && pa_channel_map_valid (map)) {
         if (pa_channel_map_can_balance (map))
             flags |= MATE_MIXER_STREAM_CONTROL_CAN_BALANCE;
         else
@@ -665,9 +684,24 @@ pulse_stream_control_get_min_volume (MateMixerStreamControl *mmsc)
 static guint
 pulse_stream_control_get_max_volume (MateMixerStreamControl *mmsc)
 {
+    MateMixerStreamControlFlags flags;
+
     g_return_val_if_fail (PULSE_IS_STREAM_CONTROL (mmsc), (guint) PA_VOLUME_MUTED);
 
-    return (guint) PA_VOLUME_UI_MAX;
+    flags = mate_mixer_stream_control_get_flags (mmsc);
+
+    /*
+     * From PulseAudio wiki:
+     * For all volumes that are > PA_VOLUME_NORM (i.e. beyond the maximum volume
+     * setting of the hw) PA will do digital amplification. This works only for
+     * devices that have PA_SINK_DECIBEL_VOLUME/PA_SOURCE_DECIBEL_VOLUME set. For
+     * devices that lack this flag do not extend the volume slider like this, it
+     * will not have any effect.
+     */
+    if (flags & MATE_MIXER_STREAM_CONTROL_HAS_DECIBEL)
+        return (guint) PA_VOLUME_UI_MAX;
+    else
+        return (guint) PA_VOLUME_NORM;
 }
 
 static guint
@@ -688,7 +722,7 @@ pulse_stream_control_get_base_volume (MateMixerStreamControl *mmsc)
     control = PULSE_STREAM_CONTROL (mmsc);
 
     if (control->priv->base_volume > 0)
-        return control->priv->base_volume;
+        return (guint) control->priv->base_volume;
     else
         return (guint) PA_VOLUME_NORM;
 }
